import React, { createContext, useState, useContext, useEffect } from 'react';
import { useColorScheme } from 'react-native';
import twColors from './twColors';
export const defaultLightColors = {
  backgroundColor: '#ffffff',
  textDefaultColor: '#222222',
  textLinkColor: '#38bdf8',
  borderColor: 'rgba(0, 0, 0, 0.1)',
  chipsItemBackgroundColor: '#f1f5f9',
  chipsItemBorderColor: '#e2e8f0',
  chipsItemCloseButtonBackgroundColor: '#e2e8f0',
  chipsItemCloseButtonIconColor: '#94a3b8',
  checkboxOff: twColors.slate[200],
  checkboxOn: twColors.blue[500]
};
export const defaultDarkColors = {
  backgroundColor: '#000000',
  textDefaultColor: '#ffffff',
  textLinkColor: '#38bdf8',
  borderColor: 'rgba(255, 255, 255, 0.25)',
  chipsItemBackgroundColor: '#334155',
  chipsItemBorderColor: '#334155',
  chipsItemCloseButtonBackgroundColor: '#94a3b8',
  chipsItemCloseButtonIconColor: '#0f172a',
  checkboxOff: twColors.slate[700],
  checkboxOn: twColors.blue[500]
};
export const defaultColors = {
  light: defaultLightColors,
  dark: defaultDarkColors
};
const defaultSizes = {
  screenPadding: 16,
  buttonHeight: 44,
  chipsItemHeight: 44,
  chipsItemBorderRadius: 12,
  chipsItemGapSize: 6,
  chipsItemPaddingLeft: 12,
  chipsItemPaddingRight: 12,
  checkboxSize: 20,
  checkboxBorderRadius: 4
};

const getInitialConfigs = function (props) {
  let scheme = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 'light';
  const baseColors = { ...defaultColors[scheme],
    ...(props === null || props === void 0 ? void 0 : props.colors)
  };
  const sizes = { ...defaultSizes,
    ...(props === null || props === void 0 ? void 0 : props.sizes)
  };
  const defaultTextStyle = [{
    color: baseColors.textDefaultColor
  }, props === null || props === void 0 ? void 0 : props.textBaseStyle];
  return {
    colors: baseColors,
    sizes: sizes,
    textBaseStyle: defaultTextStyle,
    textLinkStyle: [...defaultTextStyle, {
      color: baseColors.textLinkColor
    }, props === null || props === void 0 ? void 0 : props.textLinkStyle],
    buttonContainerStyle: [{
      height: sizes.buttonHeight,
      flexDirection: 'row',
      alignItems: 'center',
      justifyContent: 'space-between'
    }, props === null || props === void 0 ? void 0 : props.buttonContainerStyle],
    buttonTextStyle: [...defaultTextStyle, {
      fontWeight: 'bold'
    }, props === null || props === void 0 ? void 0 : props.buttonTextStyle],
    containerStyle: [{
      paddingHorizontal: sizes.screenPadding
    }, props === null || props === void 0 ? void 0 : props.containerStyle],
    chipsGridStyle: [{
      flexDirection: 'row',
      alignItems: 'center',
      flexWrap: 'wrap',
      marginLeft: -sizes.chipsItemGapSize / 2,
      marginRight: -sizes.chipsItemGapSize / 2
    }, props === null || props === void 0 ? void 0 : props.chipsGridStyle],
    chipsItemContainerStyle: [{
      flexDirection: 'row',
      justifyContent: 'center',
      alignItems: 'center',
      minHeight: sizes.chipsItemHeight,
      borderRadius: sizes.chipsItemBorderRadius,
      margin: sizes.chipsItemGapSize / 2,
      paddingLeft: sizes.chipsItemPaddingLeft,
      paddingRight: sizes.chipsItemPaddingRight,
      backgroundColor: baseColors.chipsItemBackgroundColor,
      borderWidth: 1,
      borderColor: baseColors.chipsItemBorderColor
    }, props === null || props === void 0 ? void 0 : props.chipsItemContainerStyle],
    chipsItemCloseButtonStyle: [{
      backgroundColor: baseColors.chipsItemCloseButtonBackgroundColor,
      justifyContent: 'center',
      alignItems: 'center',
      marginLeft: 8,
      aspectRatio: 1 / 1,
      padding: 8,
      borderRadius: 99
    }, props === null || props === void 0 ? void 0 : props.chipsItemCloseButtonStyle],
    chipsItemCloseIconStyle: [{
      width: 16,
      height: 16,
      tintColor: baseColors.chipsItemCloseButtonIconColor
    }, props === null || props === void 0 ? void 0 : props.chipsItemCloseIconStyle],
    chipsItemTextStyle: [...defaultTextStyle, {
      fontWeight: 'bold'
    }, props === null || props === void 0 ? void 0 : props.chipsItemCloseIconStyle],
    headerContainerStyle: [{
      height: 54,
      flexDirection: 'row',
      alignItems: 'center',
      paddingHorizontal: 16,
      borderBottomWidth: 1,
      borderBottomColor: baseColors.borderColor
    }, props === null || props === void 0 ? void 0 : props.headerContainerStyle],
    headerTextStyle: [...defaultTextStyle, {}, props === null || props === void 0 ? void 0 : props.headerTextStyle],
    headerTitleStyle: [...defaultTextStyle, {
      textAlign: 'center',
      fontWeight: 'bold'
    }, props === null || props === void 0 ? void 0 : props.headerTitleStyle],
    searchInputStyle: [{
      borderWidth: 1,
      borderColor: baseColors.borderColor,
      height: 44,
      borderRadius: 22,
      paddingHorizontal: 34
    }, props === null || props === void 0 ? void 0 : props.searchInputStyle],
    pickerItemContainerStyle: [{
      flexDirection: 'row',
      alignItems: 'center',
      padding: 16,
      opacity: 1
    }, props === null || props === void 0 ? void 0 : props.pickerItemContainerStyle],
    pickerItemTextStyle: [...defaultTextStyle, {
      flex: 1
    }, props === null || props === void 0 ? void 0 : props.pickerItemTextStyle],
    pickerItemIconStyle: [{
      width: 24,
      height: 24,
      marginRight: 8,
      tintColor: baseColors.chipsItemCloseButtonIconColor
    }, props === null || props === void 0 ? void 0 : props.pickerItemIconStyle],
    pickerItemActiveIconStyle: [{
      tintColor: baseColors.textDefaultColor
    }, props === null || props === void 0 ? void 0 : props.pickerItemActiveIconStyle],
    pickerItemDisabledStyle: [{
      opacity: 0.5
    }, props === null || props === void 0 ? void 0 : props.pickerItemDisabledStyle]
  };
};

const AsyncPickerConfigsContext = /*#__PURE__*/createContext(undefined);

const AsyncPickerConfigsProvider = _ref => {
  let {
    children,
    configParams = undefined
  } = _ref;
  const colorScheme = useColorScheme() === 'dark' ? 'dark' : 'light';
  const [configs, setConfigs] = useState(getInitialConfigs(configParams));
  useEffect(() => {
    setConfigs(getInitialConfigs(configParams, colorScheme));
  }, [configParams, colorScheme]);
  return /*#__PURE__*/React.createElement(AsyncPickerConfigsContext.Provider, {
    value: {
      configs,
      setConfigs
    }
  }, children);
};

function useAsyncPickerConfigs() {
  const context = useContext(AsyncPickerConfigsContext);

  if (context === undefined) {
    throw new Error('useCount must be used within a CountProvider');
  }

  return context;
}

export { AsyncPickerConfigsProvider, useAsyncPickerConfigs }; // export const initialConfigs = {
//   colors: {
//     backgroundColor: '#ffffff',
//     headerBackgroundColor: '#ffffff',
//     headerTextColor: '#222222',
//     linkColor: '#0ea5e9',
//     textPrimary: 'rgba(0, 0, 0, 0.87)',
//     textSecondary: 'rgba(0, 0, 0, 0.79)',
//     textMuted: 'rgba(0, 0, 0, 0.54)',
//   },
//   initialPadding: 16,
//   textBaseStyle: { color: '#222222' } as TextStyle,
//   textSecondaryStyle: { color: '#222222' } as TextStyle,
//   textMutedStyle: { color: '#222222' } as TextStyle,
//   textLinkStyle: { color: '#0ea5e9' } as TextStyle,
//   chipsTextStyle: {} as TextStyle,
// };
// const lightColors = {
//   backgroundColor: '#ffffff',
//   headerBackgroundColor: '#ffffff',
//   headerTextColor: '#222222',
//   linkColor: '#0ea5e9',
//   textPrimary: 'rgba(0, 0, 0, 0.87)',
//   textSecondary: 'rgba(0, 0, 0, 0.79)',
//   textMuted: 'rgba(0, 0, 0, 0.54)',
// };
// const darkColors: typeof lightColors = {
//   backgroundColor: '#ffffff',
//   headerBackgroundColor: '#ffffff',
//   headerTextColor: '#222222',
//   linkColor: '#0ea5e9',
//   textPrimary: 'rgba(0, 0, 0, 0.87)',
//   textSecondary: 'rgba(0, 0, 0, 0.79)',
//   textMuted: 'rgba(0, 0, 0, 0.54)',
// };
// const initialColors = {
//   light: lightColors,
//   dark: darkColors,
// };
// const initialConfigs = {
//   colors: typeof initialColors,
//   screenPadding: 16,
//   containerStyle: { backgroundColor: 'yellow', padding: 16 } as ViewStyle,
// };
//# sourceMappingURL=AsyncPickerConfigsContext.js.map