import React, { useEffect, useState } from 'react';
import {
  KeyboardAvoidingView,
  Modal,
  Platform,
  SafeAreaView,
  StatusBar,
  View,
} from 'react-native';
import AsyncPickerButton from './AsyncPickerButton';
import type { AsyncPickerModuleProps } from './AsyncPickerModule';
import AsyncPickerModule from './AsyncPickerModule';
import {
  InitialConfigsProps,
  useAsyncPickerConfigs,
} from './utils/AsyncPickerConfigsContext';

export type AsyncPickerProps<T> = AsyncPickerModuleProps<T> & {
  renderButton?: (data: T[] | undefined) => React.ReactChild;
  renderChips?: (item: T, deleteItem: (item: T) => void) => React.ReactNode;
  label?: string;
  themeConfigs?: Partial<InitialConfigsProps>;
};

const AsyncPicker = <T,>({
  onOpen,
  onClose,
  label = 'Select',
  renderChips,
  value,
  keyExtractor,
  onChange,
  themeConfigs,
  ...props
}: AsyncPickerProps<T>) => {
  const [isModalOpen, setIsModalOpen] = useState(false);
  const { configs, setConfigs } = useAsyncPickerConfigs();

  useEffect(() => {
    setConfigs({ ...configs, ...themeConfigs });
  }, [themeConfigs]);

  const onModalOpen = () => {
    setIsModalOpen(true);
    if (onOpen) {
      onOpen();
    }
  };

  const onModalClose = () => {
    setIsModalOpen(false);
    if (onClose) {
      onClose();
    }
  };

  const onDeleteItem = (item: T) => {
    console.log(item);
    const newValue = value.filter(
      (el) => keyExtractor(el) !== keyExtractor(item)
    );
    onChange(newValue);
  };

  return (
    <View style={[configs.containerStyle]}>
      <AsyncPickerButton
        label={label}
        value={value}
        onPress={() => {
          onModalOpen();
        }}
      />

      {renderChips && (
        <View style={configs.chipsGridStyle}>
          {value.map((item) => {
            return (
              <React.Fragment key={keyExtractor(item)}>
                {renderChips(item, onDeleteItem)}
              </React.Fragment>
            );
          })}
        </View>
      )}

      <KeyboardAvoidingView
        behavior={Platform.OS === 'ios' ? 'padding' : 'height'}
        style={{ flex: 1 }}
      >
        <Modal
          animationType="slide"
          presentationStyle="formSheet"
          transparent={false}
          visible={isModalOpen}
          onRequestClose={() => {
            setIsModalOpen(!isModalOpen);
          }}
        >
          <SafeAreaView style={{ flex: 1 }}>
            <StatusBar barStyle="light-content" />
            <AsyncPickerModule
              value={value}
              keyExtractor={keyExtractor}
              onChange={onChange}
              {...props}
              onClose={() => {
                onModalClose();
              }}
            />
          </SafeAreaView>
        </Modal>
      </KeyboardAvoidingView>
    </View>
  );
};

export default AsyncPicker;
