import React, { CSSProperties, useCallback, useState } from 'react';
import {
  ActivityIndicator,
  FlatList,
  FlatListProps,
  ListRenderItemInfo,
  Text,
  TextInput,
  View,
} from 'react-native';
import AsyncPickerHeader from './AsyncPickerHeader';
import { useAsyncPickerConfigs } from './utils/AsyncPickerConfigsContext';
import { isTextIncludedInString } from './utils/helpers';

interface RenderPickerItemProps<T> extends ListRenderItemInfo<T> {
  onItemPress: (item: T) => void;
  isActive: boolean;
  isDisabled: boolean;
}

export interface AsyncPickerModuleProps<T>
  extends Omit<FlatListProps<T>, 'renderItem'> {
  value: T[];
  onChange: (item: T[]) => void;
  keyExtractor: (item: T) => string;
  searchableStringExtractor?: (item: T) => string;
  limit?: number;
  renderItem: (renderProps: RenderPickerItemProps<T>) => React.ReactChild;
  onClose?: () => void;
  onOpen?: () => void;
  onSearch?: (text: string) => void;
  headerStyle?: CSSProperties;
  closeOnSelect?: boolean;
  initialConfigs?: any;
  isLoading?: boolean;
  noResultComponent?: (query: string) => React.ReactElement;
}

const AsyncPickerModule = <T,>(props: AsyncPickerModuleProps<T>) => {
  const {
    renderItem,
    data,
    value,
    searchableStringExtractor,
    onChange,
    keyExtractor,
    limit = 0,
    onClose,
    onOpen,
    onSearch,
    closeOnSelect,
    isLoading,
    noResultComponent,
    ListEmptyComponent,
    ...otherProps
  } = props;
  const { configs } = useAsyncPickerConfigs();
  const [inputValue, setInputValue] = useState('');
  const [searchResult, setSearchResult] = useState<T[]>([]);

  const onOptionPress = (option: T) => {
    if (limit !== 1) {
      if (value.find((el) => keyExtractor(el) === keyExtractor(option))) {
        onChange(
          value.filter((el) => keyExtractor(el) !== keyExtractor(option))
        );
        return;
      }
    }

    const selectedItems = limit === 1 ? [option] : [...value, option];
    if (limit && limit > 1 && selectedItems.length > limit) {
      return;
    }

    onChange(selectedItems);

    if (closeOnSelect && onClose) {
      onClose();
    }
  };

  const getOptions = useCallback(() => {
    if (searchableStringExtractor && inputValue) {
      return searchResult;
    }

    return data;
  }, [searchableStringExtractor, searchResult, data, !!inputValue]);

  return (
    <View style={{ flex: 1, backgroundColor: configs.colors.backgroundColor }}>
      <AsyncPickerHeader onSave={onClose} title="Search" />
      {(searchableStringExtractor || onSearch) && (
        <View style={{ margin: 16, justifyContent: 'center' }}>
          <TextInput
            value={inputValue}
            onChangeText={(el) => {
              setInputValue(el);
              if (onSearch) {
                onSearch(el);
              }

              if (searchableStringExtractor) {
                const res = data?.filter((item) =>
                  isTextIncludedInString(el, searchableStringExtractor(item))
                );
                setSearchResult(res || []);
              }
            }}
            placeholder="Search ..."
            style={configs.searchInputStyle}
            clearButtonMode="while-editing"
          />
          {isLoading && (
            <ActivityIndicator style={{ position: 'absolute', right: 32 }} />
          )}
        </View>
      )}

      <FlatList
        data={getOptions() || []}
        keyExtractor={keyExtractor}
        renderItem={({ item, index, separators }) => {
          const isActive = value.some(
            (el) => keyExtractor(el) === keyExtractor(item)
          );
          const isDisabled =
            !isActive && !!limit && limit > 1 && value.length === limit;

          return (
            <>
              {renderItem({
                item,
                index,
                separators,
                isActive,
                isDisabled,
                onItemPress: onOptionPress,
              })}
            </>
          );
        }}
        ListEmptyComponent={
          isLoading ? undefined : !!inputValue ? (
            noResultComponent ? (
              noResultComponent(inputValue)
            ) : (
              <View>
                <Text style={{ textAlign: 'center' }}>No results</Text>
              </View>
            )
          ) : (
            ListEmptyComponent
          )
        }
        {...otherProps}
      />
    </View>
  );
};

export default AsyncPickerModule;
