import React, { createContext, useState, useContext, useEffect } from 'react';
import {
  ImageStyle,
  StyleProp,
  TextStyle,
  useColorScheme,
  ViewStyle,
} from 'react-native';
import twColors from './twColors';

export const defaultLightColors = {
  backgroundColor: '#ffffff',
  textDefaultColor: '#222222',
  textLinkColor: '#38bdf8',
  borderColor: 'rgba(0, 0, 0, 0.1)',
  chipsItemBackgroundColor: '#f1f5f9',
  chipsItemBorderColor: '#e2e8f0',
  chipsItemCloseButtonBackgroundColor: '#e2e8f0',
  chipsItemCloseButtonIconColor: '#94a3b8',
  checkboxOff: twColors.slate[200],
  checkboxOn: twColors.blue[500],
};

export const defaultDarkColors: typeof defaultLightColors = {
  backgroundColor: '#000000',
  textDefaultColor: '#ffffff',
  textLinkColor: '#38bdf8',
  borderColor: 'rgba(255, 255, 255, 0.25)',
  chipsItemBackgroundColor: '#334155',
  chipsItemBorderColor: '#334155',
  chipsItemCloseButtonBackgroundColor: '#94a3b8',
  chipsItemCloseButtonIconColor: '#0f172a',
  checkboxOff: twColors.slate[700],
  checkboxOn: twColors.blue[500],
};

export const defaultColors = {
  light: defaultLightColors,
  dark: defaultDarkColors,
};

export type ColorsProps = typeof defaultLightColors;

const defaultSizes = {
  screenPadding: 16,
  buttonHeight: 44,

  chipsItemHeight: 44,
  chipsItemBorderRadius: 12,
  chipsItemGapSize: 6,
  chipsItemPaddingLeft: 12,
  chipsItemPaddingRight: 12,

  checkboxSize: 20,
  checkboxBorderRadius: 4,
};

export interface InitialConfigsProps {
  colors: ColorsProps;
  sizes: typeof defaultSizes;
  textBaseStyle: StyleProp<TextStyle>;
  textLinkStyle: StyleProp<TextStyle>;
  buttonContainerStyle: StyleProp<ViewStyle>;
  buttonTextStyle: StyleProp<TextStyle>;
  containerStyle: StyleProp<ViewStyle>;
  chipsGridStyle: StyleProp<ViewStyle>;
  chipsItemContainerStyle: StyleProp<ViewStyle>;
  chipsItemCloseButtonStyle: StyleProp<ViewStyle>;
  chipsItemCloseIconStyle: StyleProp<ImageStyle>;
  chipsItemTextStyle: StyleProp<TextStyle>;
  headerContainerStyle: StyleProp<ViewStyle>;
  headerTextStyle: StyleProp<TextStyle>;
  headerTitleStyle: StyleProp<TextStyle>;
  searchInputStyle: StyleProp<ViewStyle>;
  pickerItemContainerStyle: StyleProp<ViewStyle>;
  pickerItemTextStyle: StyleProp<TextStyle>;
  pickerItemIconStyle: StyleProp<ImageStyle>;
  pickerItemActiveIconStyle: StyleProp<ImageStyle>;
  pickerItemDisabledStyle: StyleProp<ViewStyle>;
}

const getInitialConfigs = (
  props: InitialConfigsProps | undefined,
  scheme: 'dark' | 'light' = 'light'
): InitialConfigsProps => {
  const baseColors = { ...defaultColors[scheme], ...props?.colors };
  const sizes = { ...defaultSizes, ...props?.sizes };
  const defaultTextStyle: StyleProp<TextStyle> = [
    { color: baseColors.textDefaultColor },
    props?.textBaseStyle,
  ];

  return {
    colors: baseColors,
    sizes: sizes,
    textBaseStyle: defaultTextStyle,

    textLinkStyle: [
      ...defaultTextStyle,
      { color: baseColors.textLinkColor },
      props?.textLinkStyle,
    ],

    buttonContainerStyle: [
      {
        height: sizes.buttonHeight,
        flexDirection: 'row',
        alignItems: 'center',
        justifyContent: 'space-between',
      },
      props?.buttonContainerStyle,
    ],

    buttonTextStyle: [
      ...defaultTextStyle,
      {
        fontWeight: 'bold',
      },
      props?.buttonTextStyle,
    ],

    containerStyle: [
      { paddingHorizontal: sizes.screenPadding },
      props?.containerStyle,
    ],

    chipsGridStyle: [
      {
        flexDirection: 'row',
        alignItems: 'center',
        flexWrap: 'wrap',
        marginLeft: -sizes.chipsItemGapSize / 2,
        marginRight: -sizes.chipsItemGapSize / 2,
      },
      props?.chipsGridStyle,
    ],

    chipsItemContainerStyle: [
      {
        flexDirection: 'row',
        justifyContent: 'center',
        alignItems: 'center',
        minHeight: sizes.chipsItemHeight,
        borderRadius: sizes.chipsItemBorderRadius,
        margin: sizes.chipsItemGapSize / 2,
        paddingLeft: sizes.chipsItemPaddingLeft,
        paddingRight: sizes.chipsItemPaddingRight,
        backgroundColor: baseColors.chipsItemBackgroundColor,
        borderWidth: 1,
        borderColor: baseColors.chipsItemBorderColor,
      },
      props?.chipsItemContainerStyle,
    ],

    chipsItemCloseButtonStyle: [
      {
        backgroundColor: baseColors.chipsItemCloseButtonBackgroundColor,
        justifyContent: 'center',
        alignItems: 'center',
        marginLeft: 8,
        aspectRatio: 1 / 1,
        padding: 8,
        borderRadius: 99,
      },
      props?.chipsItemCloseButtonStyle,
    ],

    chipsItemCloseIconStyle: [
      {
        width: 16,
        height: 16,
        tintColor: baseColors.chipsItemCloseButtonIconColor,
      },
      props?.chipsItemCloseIconStyle,
    ],

    chipsItemTextStyle: [
      ...defaultTextStyle,
      {
        fontWeight: 'bold',
      },
      props?.chipsItemCloseIconStyle,
    ],

    headerContainerStyle: [
      {
        height: 54,
        flexDirection: 'row',
        alignItems: 'center',
        paddingHorizontal: 16,
        borderBottomWidth: 1,
        borderBottomColor: baseColors.borderColor,
      },
      props?.headerContainerStyle,
    ],
    headerTextStyle: [...defaultTextStyle, {}, props?.headerTextStyle],
    headerTitleStyle: [
      ...defaultTextStyle,
      { textAlign: 'center', fontWeight: 'bold' },
      props?.headerTitleStyle,
    ],
    searchInputStyle: [
      {
        borderWidth: 1,
        borderColor: baseColors.borderColor,
        height: 44,
        borderRadius: 22,
        paddingHorizontal: 34,
      },
      props?.searchInputStyle,
    ],

    pickerItemContainerStyle: [
      {
        flexDirection: 'row',
        alignItems: 'center',
        padding: 16,
        opacity: 1,
      },
      props?.pickerItemContainerStyle,
    ],
    pickerItemTextStyle: [
      ...defaultTextStyle,
      { flex: 1 },
      props?.pickerItemTextStyle,
    ],
    pickerItemIconStyle: [
      {
        width: 24,
        height: 24,
        marginRight: 8,
        tintColor: baseColors.chipsItemCloseButtonIconColor,
      },
      props?.pickerItemIconStyle,
    ],
    pickerItemActiveIconStyle: [
      { tintColor: baseColors.textDefaultColor },
      props?.pickerItemActiveIconStyle,
    ],
    pickerItemDisabledStyle: [{ opacity: 0.5 }, props?.pickerItemDisabledStyle],
  };
};

export interface AsyncPickerConfigsContextProps {
  configs: InitialConfigsProps;
  setConfigs: (configs: InitialConfigsProps) => void;
}

const AsyncPickerConfigsContext = createContext<
  AsyncPickerConfigsContextProps | undefined
>(undefined);

const AsyncPickerConfigsProvider = ({
  children,
  configParams = undefined,
}: {
  children: React.ReactNode;
  configParams?: InitialConfigsProps;
}) => {
  const colorScheme = useColorScheme() === 'dark' ? 'dark' : 'light';
  const [configs, setConfigs] = useState(getInitialConfigs(configParams));

  useEffect(() => {
    setConfigs(getInitialConfigs(configParams, colorScheme));
  }, [configParams, colorScheme]);

  return (
    <AsyncPickerConfigsContext.Provider
      value={
        {
          configs,
          setConfigs,
        } as AsyncPickerConfigsContextProps
      }
    >
      {children}
    </AsyncPickerConfigsContext.Provider>
  );
};

function useAsyncPickerConfigs() {
  const context = useContext(AsyncPickerConfigsContext);

  if (context === undefined) {
    throw new Error('useCount must be used within a CountProvider');
  }

  return context;
}

export { AsyncPickerConfigsProvider, useAsyncPickerConfigs };

// export const initialConfigs = {
//   colors: {
//     backgroundColor: '#ffffff',
//     headerBackgroundColor: '#ffffff',
//     headerTextColor: '#222222',
//     linkColor: '#0ea5e9',

//     textPrimary: 'rgba(0, 0, 0, 0.87)',
//     textSecondary: 'rgba(0, 0, 0, 0.79)',
//     textMuted: 'rgba(0, 0, 0, 0.54)',
//   },

//   initialPadding: 16,
//   textBaseStyle: { color: '#222222' } as TextStyle,
//   textSecondaryStyle: { color: '#222222' } as TextStyle,
//   textMutedStyle: { color: '#222222' } as TextStyle,
//   textLinkStyle: { color: '#0ea5e9' } as TextStyle,

//   chipsTextStyle: {} as TextStyle,
// };

// const lightColors = {
//   backgroundColor: '#ffffff',
//   headerBackgroundColor: '#ffffff',
//   headerTextColor: '#222222',
//   linkColor: '#0ea5e9',

//   textPrimary: 'rgba(0, 0, 0, 0.87)',
//   textSecondary: 'rgba(0, 0, 0, 0.79)',
//   textMuted: 'rgba(0, 0, 0, 0.54)',
// };

// const darkColors: typeof lightColors = {
//   backgroundColor: '#ffffff',
//   headerBackgroundColor: '#ffffff',
//   headerTextColor: '#222222',
//   linkColor: '#0ea5e9',

//   textPrimary: 'rgba(0, 0, 0, 0.87)',
//   textSecondary: 'rgba(0, 0, 0, 0.79)',
//   textMuted: 'rgba(0, 0, 0, 0.54)',
// };

// const initialColors = {
//   light: lightColors,
//   dark: darkColors,
// };

// const initialConfigs = {
//   colors: typeof initialColors,
//   screenPadding: 16,
//   containerStyle: { backgroundColor: 'yellow', padding: 16 } as ViewStyle,
// };
